package com.ruoyi.framework.util;

import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.domain.BaseEntity;
import com.ruoyi.common.core.domain.entity.SysRole;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.text.Convert;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.framework.security.context.PermissionContextHolder;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author wjh
 * 2024/6/13
 * 更新时间 2024-10-18
 */
public class DataScopeUtil {

    /**
     * 数据权限过滤关键字
     */
    public static final String DATA_SCOPE = "dataScope";

    /**
     * 全部数据权限
     */
    public static final String DATA_SCOPE_ALL = "1";

    /**
     * 自定数据权限
     */
    public static final String DATA_SCOPE_CUSTOM = "2";

    /**
     * 部门数据权限
     */
    public static final String DATA_SCOPE_DEPT = "3";

    /**
     * 部门及以下数据权限
     */
    public static final String DATA_SCOPE_DEPT_AND_CHILD = "4";

    /**
     * 仅本人数据权限
     */
    public static final String DATA_SCOPE_SELF = "5";

    public static String dataScope(String deptAlias, String userAlias, boolean needScope) {
        if (needScope) {
            // 获取当前的用户
            LoginUser loginUser = SecurityUtils.getLoginUser();
            if (StringUtils.isNotNull(loginUser))
            {
                SysUser currentUser = loginUser.getUser();
                // 如果是超级管理员，则不过滤数据
                if (StringUtils.isNotNull(currentUser) && !currentUser.isAdmin())
                {
                    StringBuilder sqlString = getSqlString(currentUser, deptAlias, userAlias, PermissionContextHolder.getContext());
                    if (StringUtils.isNotBlank(sqlString.toString())) {
                        return getJoinSqlString(sqlString, "");
                    }
                }
            }
        }
        return "";
    }

    public static StringBuilder getSqlString(SysUser user, String deptAlias, String userAlias, String permission) {
        List<String> deptList = StringUtils.hasText(deptAlias) ? Arrays.asList(deptAlias.split(",")) : new ArrayList<>();
        List<String> userList = StringUtils.hasText(userAlias) ? Arrays.asList(userAlias.split(",")) : new ArrayList<>();
        StringBuilder sqlString = new StringBuilder();
        List<String> conditions = new ArrayList<String>();
        List<String> scopeCustomIds = new ArrayList<String>();
        user.getRoles().forEach(role -> {
            if (DATA_SCOPE_CUSTOM.equals(role.getDataScope()) && StringUtils.equals(role.getStatus(), UserConstants.ROLE_NORMAL) && StringUtils.containsAny(role.getPermissions(), Convert.toStrArray(permission)))
            {
                scopeCustomIds.add(Convert.toStr(role.getRoleId()));
            }
        });

        // 多个角色，逐一判断权限
        for (SysRole role : user.getRoles()) {
            String dataScope = role.getDataScope();
            if (conditions.contains(dataScope) || StringUtils.equals(role.getStatus(), UserConstants.ROLE_DISABLE)) {
                continue;
            }
            if (!StringUtils.containsAny(role.getPermissions(), Convert.toStrArray(permission))) {
                continue;
            }
            // 全部数据范围
            if (DATA_SCOPE_ALL.equals(dataScope)) {
                sqlString = new StringBuilder();
                conditions.add(dataScope);
                break;
            }
            // 自定义数据范围
            else if (DATA_SCOPE_CUSTOM.equals(dataScope)) {
                if (scopeCustomIds.size() > 1) {
                    // 多个自定数据权限使用in查询，避免多次拼接。
                    sqlString.append(getForeachSql(" OR {}.dept_id IN ( SELECT dept_id FROM sys_role_dept WHERE role_id in ({}) ) ", deptList,  String.join(",", scopeCustomIds)));
                } else {
                    sqlString.append(getForeachSql(" OR {}.dept_id IN ( SELECT dept_id FROM sys_role_dept WHERE role_id = {} ) ", deptList, role.getRoleId()));
                }
            }
            // 仅本部门
            else if (DATA_SCOPE_DEPT.equals(dataScope)) {
                sqlString.append(getForeachSql(" OR {}.dept_id = {} ", deptList, user.getDeptId()));
            }
            // 本部门及下级部门
            else if (DATA_SCOPE_DEPT_AND_CHILD.equals(dataScope)) {
                sqlString.append(getForeachSql(" OR {}.dept_id IN ( SELECT dept_id FROM sys_dept WHERE dept_id = {} or find_in_set( {} , ancestors ) )", deptList, user.getDeptId(), user.getDeptId()));
            }
            // 仅本人
            else if (DATA_SCOPE_SELF.equals(dataScope)) {
                if (CollectionUtils.isNotEmpty(userList)) {
                    sqlString.append(getForeachSql(" OR {}.user_id = {} ", userList, user.getUserId()));
                } else {
                    // 数据权限为仅本人且没有userAlias别名不查询任何数据
                    sqlString.append(" OR 1=0 ");
                }
            }
            conditions.add(dataScope);
        }

        // 角色都不包含传递过来的权限字符，这个时候sqlString也会为空，所以要限制一下,不查询任何数据
        if (StringUtils.isEmpty(conditions)) {
            sqlString.append(" OR 1=0 ");
        }

        return sqlString;
    }

    /**
     * 获取列表循环SQL
     */
    private static String getForeachSql(String template, List<String> list, Object ...others) {
        StringBuilder sb = new StringBuilder();
        for (String s : list) {
            // 创建一个新的数组，包含 s 和 others 的所有元素
            Object[] params = new Object[others.length + 1];
            params[0] = s; // 将 s 放在第一个位置
            System.arraycopy(others, 0, params, 1, others.length); // 将 others 中的元素依次放入新数组中
            sb.append(StringUtils.format(template, params));
        }
        return sb.toString();
    }

    public static void joinSqlString(BaseEntity baseEntity, StringBuilder sqlString) {
        String scope = (String) baseEntity.getParams().get(DATA_SCOPE);
        baseEntity.getParams().put(DATA_SCOPE, getJoinSqlString(sqlString, scope));
    }

    private static String getJoinSqlString(StringBuilder sqlString, String scope) {
        return " AND (" + scope + sqlString.substring(4) + ")";
    }

}
