package com.ruoyi.yh.prodOrder.service.impl;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.alibaba.fastjson2.JSONArray;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.k3cloud.constants.fileds.K3ProdField;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.yh.material.domain.MaterialVO;
import com.ruoyi.yh.material.service.MaterialService;
import com.ruoyi.yh.prodOrder.domain.ProdOrder;
import com.ruoyi.yh.prodOrder.service.ProdOrderConverter;
import com.ruoyi.yh.unit.domain.UnitVO;
import com.ruoyi.yh.unit.service.UnitService;

/**
 * @author wjh
 * 2024/10/30
 */
@Service
public class ProdOrderConverterImpl implements ProdOrderConverter {

    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private MaterialService materialService;

    @Autowired
    private UnitService unitService;


    /**
     * 将ERP数据转为PO列表
     *
     * @param headers 表头列表
     * @param erpList ERP数据列表
     */
    @Override
    public List<ProdOrder> toPoByErpList(List<String> headers, List<JSONArray> erpList) {
        if (CollectionUtils.isEmpty(headers) || CollectionUtils.isEmpty(erpList)) {
            return Collections.emptyList();
        }

        // 创建erpList大小的对象列表
        List<ProdOrder> result = new ArrayList<>(erpList.size());

        LocalDateTime syncTime = LocalDateTime.now();

        Set<String> erpWorkShopIds = new HashSet<>(); // ERP生产车间ID
        Set<String> erpMaterialIds = new HashSet<>(); // ERP物料ID
        Set<String> erpUnitIds = new HashSet<>();   // ERP单位ID

        for (JSONArray row : erpList) {
            ProdOrder po = new ProdOrder();
            for (int i = 0; i < headers.size(); i++) {
                String header = headers.get(i);
                switch(header) {
                    case K3ProdField.FID:
                        po.setErpId(row.getString(i));
                        break;
                    case K3ProdField.F_BILL_NO:
                        po.setErpBillNo(row.getString(i));
                        break;
                    case K3ProdField.F_DOCUMENT_STATUS:
                        po.setErpDocumentStatus(row.getString(i));
                        break;
                    case K3ProdField.F_CREATE_DATE:
                        po.setErpCreateDate(DateUtils.toLocalDateTime(row.getDate(i)));
                        break;
                    case K3ProdField.F_MODIFY_DATE:
                        po.setErpModifyDate(DateUtils.toLocalDateTime(row.getDate(i)));
                        break;
                    case K3ProdField.F_DESCRIPTION:
                        po.setErpDescription(row.getString(i));
                        break;
                    case K3ProdField.F_DATE:
                        po.setErpDate(DateUtils.toLocalDateTime(row.getDate(i)));
                        break;
                    case K3ProdField.F_IS_REWORK:
                        po.setErpIsRework(row.getBoolean(i));
                        break;
                    case K3ProdField.F_ROW_ID:
                        po.setErpRowId(row.getString(i));
                        break;
                    case K3ProdField.F_MEMO_ITEM:
                        po.setErpMemoItem(row.getString(i));
                        break;
                    case K3ProdField.F_CONVEY_DATE:
                        po.setErpConveyDate(DateUtils.toLocalDateTime(row.getDate(i)));
                        break;
                    case K3ProdField.F_STATUS:
                        po.setErpStatus(row.getString(i));
                        break;
                    case K3ProdField.F_WORK_SHOP_ID:
                        String workShopId = row.getString(i);
                        po.setErpWorkShopId(workShopId);
                        erpWorkShopIds.add(workShopId);
                        break;
                    case K3ProdField.F_REQ_SRC:
                        po.setErpReqSrc(row.getString(i));
                        break;
                    case K3ProdField.F_BASE_UNIT_QTY:
                        po.setErpBaseUnitQty(row.getBigDecimal(i));
                        break;
                    case K3ProdField.F_QTY:
                        po.setErpQty(row.getBigDecimal(i));
                        break;
                    case K3ProdField.F_UNIT_ID:
                        String unitId = row.getString(i);
                        po.setErpUnitId(unitId);
                        erpUnitIds.add(unitId);
                        break;
                    case K3ProdField.F_BASE_UNIT_ID:
                        String baseUnitId = row.getString(i);
                        po.setErpBaseUnitId(baseUnitId);
                        erpUnitIds.add(baseUnitId);
                        break;
                    case K3ProdField.F_MATERIAL_ID:
                        String materialId = row.getString(i);
                        po.setErpMaterialId(materialId);
                        erpMaterialIds.add(materialId);
                        break;
                    case K3ProdField.F_NO_STOCK_IN_QTY:
                        po.setErpNoStockInQty(row.getBigDecimal(i));
                        break;
                    case K3ProdField.F_BASE_NO_STOCK_IN_QTY:
                        po.setErpBaseNoStockInQty(row.getBigDecimal(i));
                        break;
                    case K3ProdField.FMTONO:    
                        po.setErpMtoNo(row.getString(i));
                        break;
                    case K3ProdField.FAUXPROPID:
                        po.setFauxPropId(row.getString(i));
                        break;
                    case K3ProdField.FAUXPROP:
                        po.setFauxProp(row.getString(i));
                        break;
                    default: break;
                }
            }

            // 同步时间
            po.setSyncTime(syncTime);

            result.add(po);
        }

        // 生产部门ID
        List<SysDept> deptList = deptService.selectByErpIds(erpWorkShopIds);
        for (ProdOrder prodOrder : result) {
            SysDept dept = deptList.stream().filter(item -> Objects.equals(item.getErpId(), prodOrder.getErpWorkShopId())).findFirst().orElse(null);
            if (dept != null) {
                prodOrder.setDeptId(dept.getDeptId());
            }
        }

        // 物料信息
        List<MaterialVO> materialList = materialService.selectByErpIds(erpMaterialIds);
        for (ProdOrder prodOrder : result) {
            MaterialVO material = materialList.stream().filter(item -> Objects.equals(item.getErpId(), prodOrder.getErpMaterialId())).findFirst().orElse(null);
            if (material != null) {
                prodOrder.setMaterialCategory(material.getCategory());
                prodOrder.setMaterialCover(material.getCover());
                prodOrder.setMaterialSize(material.getSize());
                prodOrder.setMaterialNumber(material.getErpNumber());
                prodOrder.setMaterialSurface(material.getSurface());
                prodOrder.setMaterialGraphics(material.getGraphics());
            } else {
                prodOrder.setMaterialCategory("");
                prodOrder.setMaterialCover("");
                prodOrder.setMaterialSize("");
                prodOrder.setMaterialNumber("");
                prodOrder.setMaterialSurface("");
                prodOrder.setMaterialGraphics("");
            }
        }

        // 单位信息
        List<UnitVO> unitList = unitService.selectByErpIds(erpUnitIds);
        for (ProdOrder prodOrder : result) {
            UnitVO unit = unitList.stream().filter(item -> Objects.equals(item.getErpId(), prodOrder.getErpUnitId())).findFirst().orElse(null);
            if (unit != null) {
                prodOrder.setUnitName(unit.getErpName());
            }

            UnitVO baseUnit = unitList.stream().filter(item -> Objects.equals(item.getErpId(), prodOrder.getErpBaseUnitId())).findFirst().orElse(null);
            if (baseUnit != null) {
                prodOrder.setBaseUnitName(baseUnit.getErpName());
            }
        }

        return result;
    }
}
