package com.ruoyi.yh.prodOrder.service.impl;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import com.alibaba.fastjson2.JSONArray;
import com.github.pagehelper.PageHelper;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.k3cloud.constants.K3FormIds;
import com.ruoyi.common.k3cloud.constants.fileds.K3ProdField;
import com.ruoyi.common.k3cloud.service.K3Service;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.yh.logImport.domain.LogImportVO;
import com.ruoyi.yh.logImport.domain.enums.LogImportBizType;
import com.ruoyi.yh.logImport.domain.enums.LogImportType;
import com.ruoyi.yh.logImport.service.LogImportService;
import com.ruoyi.yh.logImportDetail.domain.enums.LogImportDetailType;
import com.ruoyi.yh.logImportDetail.service.ILogImportDetailService;
import com.ruoyi.yh.prodOrder.domain.ProdOrder;
import com.ruoyi.yh.prodOrder.domain.ProdOrderQuery;
import com.ruoyi.yh.prodOrder.domain.ProdOrderVO;
import com.ruoyi.yh.prodOrder.domain.dto.ProdOrderSyncQuery;
import com.ruoyi.yh.prodOrder.mapper.ProdOrderMapper;
import com.ruoyi.yh.prodOrder.service.ProdOrderConverter;
import com.ruoyi.yh.prodOrder.service.ProdOrderService;

import lombok.extern.slf4j.Slf4j;

/**
 * 生产订单Service业务层处理
 *
 * @author ruoyi
 * @date 2024-10-29
 */
@Slf4j
@Service
public class ProdOrderServiceImpl implements ProdOrderService
{
    @Autowired
    private ProdOrderMapper prodOrderMapper;

    @Autowired
    private K3Service k3Service;

    @Autowired
    private ProdOrderConverter prodOrderConverter;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private LogImportService logImportService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ILogImportDetailService logImportDetailService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询生产订单
     *
     * @param id 生产订单主键
     * @return 生产订单
     */
    @Override
    public ProdOrderVO selectProdOrderById(Long id)
    {
        return prodOrderMapper.selectProdOrderById(id);
    }

    /**
     * 查询生产订单列表
     *
     * @param prodOrder 生产订单
     * @return 生产订单
     */
    @Override
    @DataScope(deptAlias = "sd")
    public List<ProdOrderVO> selectProdOrderList(ProdOrderQuery prodOrder)
    {
        return prodOrderMapper.selectProdOrderList(prodOrder);
    }

    /**
     * 新增生产订单
     *
     * @param prodOrder 生产订单
     * @return 结果
     */
    @Override
    public int insertProdOrder(ProdOrder prodOrder)
    {
        prodOrder.setCreateTime(DateUtils.getNowDate());
        return prodOrderMapper.insertProdOrder(prodOrder);
    }

    /**
     * 修改生产订单
     *
     * @param prodOrder 生产订单
     * @return 结果
     */
    @Override
    public int updateProdOrder(ProdOrder prodOrder)
    {
        return prodOrderMapper.updateProdOrder(prodOrder);
    }

    /**
     * 批量删除生产订单
     *
     * @param ids 需要删除的生产订单主键
     * @return 结果
     */
    @Override
    public int deleteProdOrderByIds(Long[] ids)
    {
        return prodOrderMapper.deleteProdOrderByIds(ids);
    }

    /**
     * 删除生产订单信息
     *
     * @param id 生产订单主键
     * @return 结果
     */
    @Override
    public int deleteProdOrderById(Long id)
    {
        return prodOrderMapper.deleteProdOrderById(id);
    }

    @Override
    public void sync(ProdOrderSyncQuery query, Long operaId) {
        // 新增一个日志
        LogImportVO importLog = logImportService.addDefaultLog(LogImportType.SYNC, LogImportBizType.PROD_ORDER, operaId);
        ServiceUtil.assertion(importLog == null, "创建同步日志失败");
        Long logId = importLog.getLogId();

        scheduledExecutorService.schedule(() -> {
            // 查询字段
            List<String> fieldKeys = Arrays.asList(
                    K3ProdField.FID,
                    K3ProdField.F_BILL_NO,
                    K3ProdField.F_DOCUMENT_STATUS,
                    K3ProdField.F_CREATE_DATE,
                    K3ProdField.F_MODIFY_DATE,
                    K3ProdField.F_DESCRIPTION,
                    K3ProdField.F_DATE,
                    K3ProdField.F_IS_REWORK,
                    K3ProdField.F_ROW_ID,
                    K3ProdField.F_MEMO_ITEM,
                    K3ProdField.F_CONVEY_DATE,
                    K3ProdField.F_STATUS,
                    K3ProdField.F_WORK_SHOP_ID,
                    K3ProdField.F_REQ_SRC,
                    K3ProdField.F_BASE_UNIT_QTY,
                    K3ProdField.F_QTY,
                    K3ProdField.F_UNIT_ID,
                    K3ProdField.F_BASE_UNIT_ID,
                    K3ProdField.F_MATERIAL_ID,
                    K3ProdField.F_NO_STOCK_IN_QTY,
                    K3ProdField.F_BASE_NO_STOCK_IN_QTY,
                    K3ProdField.FMTONO,
                    K3ProdField.FAUXPROPID,
                    K3ProdField.FAUXPROP
            );
            int startRow = 0;
            int limit = 10000;
            int size = 0;

            // 构建过滤条件
            StringBuilder fb = new StringBuilder();
            if (CollectionUtils.isNotEmptyElement(query.getStatusList())) {
                fb.append(" and FStatus in (").append(query.getStatusList().stream().map(Object::toString).collect(Collectors.joining(","))).append(")");
            }
            if (query.getId() != null) {
                fb.append(" and FID = ").append(query.getId());
            }
            String filterString = fb.substring(4);

            // 一直查询，直到查到的数据量比limit小，说明查完了
            while(true) {
                // 查询ERP数据
                List<JSONArray> erpList = k3Service.selectList(K3FormIds.PROD_ORDER, fieldKeys, startRow, limit, filterString, "");

                // 转为PO
                List<ProdOrder> prodOrderList = prodOrderConverter.toPoByErpList(fieldKeys, erpList);

                // 保存
                for (ProdOrder prodOrder : prodOrderList) {
                    scheduledExecutorService.schedule(() -> {
                        this.saveByErpRowId(prodOrder, logId);
                    }, 0, TimeUnit.SECONDS);
                }

                size += prodOrderList.size();

                if (prodOrderList.size() < limit) {
                    logImportService.updateLogToFinished(logId);
                    break;
                } else {
                    startRow += limit;
                }
            }

            return size;
        }, 0, TimeUnit.SECONDS);
    }

    /**
     * 根据单据编号查询单据
     *
     * @param billNo
     */
    @Override
    public ProdOrderVO selectByBillNo(String billNo) {
        if (StringUtils.isBlank(billNo)) {
            return null;
        }
        ProdOrderQuery query = new ProdOrderQuery();
        query.setEqErpBillNo(billNo);
        return this.selectOne(query);
    }

    /**
     * 查询单个
     *
     * @param query
     * @return
     */
    @Override
    public ProdOrderVO selectOne(ProdOrderQuery query) {
        PageHelper.startPage(1, 1 );
        List<ProdOrderVO> list = this.selectProdOrderList(query);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }

    /**
     * 增加已审核的基本单位数量
     *
     * @param orderId 订单ID
     * @param num     增加数量
     */
    @Override
    public int addVerifiedBaseNum(Long orderId, BigDecimal num) {
        if (orderId == null || num == null) {
            return 0;
        }
        return prodOrderMapper.addVerifiedBaseNum(orderId, num);
    }

    @Override
    public List<ProdOrderVO> selectSimpleList(ProdOrderQuery query,  String ...fields) {
        if (fields == null) {
            return Collections.emptyList();
        }
        return prodOrderMapper.selectSimpleList(query, Arrays.asList(fields));
    }

    /**
     * 新增或修改
     */
    private int saveByErpRowId(ProdOrder prodOrder, Long logId) {
        if (prodOrder == null || StringUtils.isBlank(prodOrder.getErpRowId())) {
            throw new ServiceException("参数错误");
        }
        String lockKey = prodOrder.getErpRowId();
        ServiceUtil.assertion(!redisLock.lock(RedisLockKey.SYNC_PROD_BILL, lockKey), "当前生产订单正在处理中，请稍后重试");

        try {
            // 若数据库中没有,则新增,否则修改
            ProdOrderVO old = this.selectByErpRowId(prodOrder.getErpRowId());
            Integer result = transactionTemplate.execute(status -> {
                if (old == null) {
                    int insert = this.insertProdOrder(prodOrder);
                    ServiceUtil.assertion(insert != 1, "新增失败");
                    int log = logImportDetailService.addResult(logId, LogImportDetailType.ADD, prodOrder, null, prodOrder.getId());
                    ServiceUtil.assertion(log != 1, "新增日志失败");
                    return insert;
                } else {
                    prodOrder.setId(old.getId());
                    int update = this.updateProdOrder(prodOrder);
                    ServiceUtil.assertion(update != 1, "修改失败");
                    int log = logImportDetailService.addResult(logId, LogImportDetailType.UPDATE, prodOrder, null, prodOrder.getId());
                    ServiceUtil.assertion(log != 1, "新增日志失败");
                    return update;
                }
            });

            return result == null ? 0 : result;
        } catch (Exception e) {
            logImportDetailService.addResult(logId, LogImportDetailType.FAIL, prodOrder, e.getMessage(), prodOrder.getId());
            return 0;
        } finally {
            redisLock.unlock(RedisLockKey.SYNC_PROD_BILL, lockKey);
        }
    }

    private ProdOrderVO selectByErpRowId(String erpRowId) {
        if (StringUtils.isBlank(erpRowId)) {
            return null;
        }
        ProdOrderQuery query = new ProdOrderQuery();
        query.setEqErpRowId(erpRowId);
        return selectOne(query);
    }

    private ProdOrderVO selectByErpId(String erpId) {
        ProdOrderQuery query = new ProdOrderQuery();
        query.setEqErpId(erpId);
        return this.selectOne(query);
    }
}
