package com.ruoyi.yh.report.service.impl;

import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.common.utils.collection.DiffListVO;
import com.ruoyi.system.service.ISysUserService;
import com.ruoyi.yh.prodOrder.service.ProdOrderService;
import com.ruoyi.yh.report.domain.Report;
import com.ruoyi.yh.report.domain.ReportQuery;
import com.ruoyi.yh.report.domain.ReportVO;
import com.ruoyi.yh.report.domain.bo.ReportBO;
import com.ruoyi.yh.report.domain.dto.ReportVerifyDTO;
import com.ruoyi.yh.report.domain.enums.ReportStatus;
import com.ruoyi.yh.report.mapper.ReportMapper;
import com.ruoyi.yh.report.service.ReportAssembler;
import com.ruoyi.yh.report.service.ReportService;
import com.ruoyi.yh.report.service.ReportValidator;
import com.ruoyi.yh.report.utils.ReportUtils;
import com.ruoyi.yh.reportOrderProd.domain.ReportOrderProd;
import com.ruoyi.yh.reportOrderProd.domain.ReportOrderProdVO;
import com.ruoyi.yh.reportOrderProd.domain.bo.ReportOrderProdBO;
import com.ruoyi.yh.reportOrderProd.service.ReportOrderProdService;
import com.ruoyi.yh.reportProd.domain.ReportProd;
import com.ruoyi.yh.reportProd.domain.ReportProdVO;
import com.ruoyi.yh.reportProd.domain.bo.ReportProdBO;
import com.ruoyi.yh.reportProd.service.ReportProdService;
import com.ruoyi.yh.reportUserProd.domain.ReportUserProd;
import com.ruoyi.yh.reportUserProd.domain.ReportUserProdVO;
import com.ruoyi.yh.reportUserProd.domain.bo.ReportUserProdBO;
import com.ruoyi.yh.reportUserProd.service.ReportUserProdService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 报表Service业务层处理
 *
 * @author ruoyi
 * @date 2024-10-18
 */
@Service
public class ReportServiceImpl implements ReportService
{
    @Autowired
    private ReportMapper reportMapper;

    @Autowired
    private TransactionTemplate transactionTemplate;

    @Autowired
    private ReportProdService reportProdService;

    @Autowired
    private ReportUserProdService reportUserProdService;

    @Autowired
    private ReportOrderProdService reportOrderProdService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ReportValidator reportValidator;

    @Autowired
    private ProdOrderService prodOrderService;

    @Autowired
    private ReportAssembler reportAssembler;

    /**
     * 查询报表
     *
     * @param reportId 报表主键
     * @return 报表
     */
    @Override
    public ReportVO selectReportByReportId(Long reportId)
    {
        return reportMapper.selectReportByReportId(reportId);
    }

    /**
     * 查询报表列表
     *
     * @param report 报表
     * @return 报表
     */
    @Override
    public List<ReportVO> selectReportList(ReportQuery report)
    {
        return reportMapper.selectReportList(report);
    }

    /**
     * 新增报表
     *
     * @param report 报表
     * @return 结果
     */
    @Override
    public int insertReport(Report report)
    {
        report.setCreateTime(DateUtils.getNowDate());
        return reportMapper.insertReport(report);
    }

    /**
     * 修改报表
     *
     * @param data 报表
     * @return 结果
     */
    @Override
    public int updateReport(Report data) {
        if (data.getReportId() == null) {
            return 0;
        }
        data.setUpdateTime(DateUtils.getNowDate());
        ReportQuery query = new ReportQuery();
        query.setReportId(data.getReportId());
        query.setStatusList(ReportStatus.canEdit());
        return this.updateByQuery(data, query);
    }

    /**
     * 批量删除报表
     *
     * @param reportIds 需要删除的报表主键
     * @return 结果
     */
    @Override
    public int deleteReportByReportIds(Long[] reportIds)
    {
        return reportMapper.deleteReportByReportIds(reportIds);
    }

    /**
     * 删除报表信息
     *
     * @param reportId 报表主键
     * @return 结果
     */
    @Override
    public int deleteReportByReportId(Long reportId)
    {
        return reportMapper.deleteReportByReportId(reportId);
    }

    @Override
    public int addReport(ReportBO bo) {
        // 校验
        reportValidator.checkPreEdit(bo);

        Integer result = transactionTemplate.execute(status -> {
            // 新增主表
            int insert = this.insertReport(bo);
            ServiceUtil.assertion(insert != 1, "新增报表失败");

            // 更新产量表
            this.batchUpdateProductList(bo);

            // 更新员工产量表
            this.batchUpdateUserProductList(bo);

            // 更新订单产量表
            this.batchUpdateOrderProductList(bo);

            // 后校验
            ReportVO vo = this.selectReportByReportId(bo.getReportId());
            reportAssembler.assembleProductList(Collections.singletonList(vo), true, true);
            reportValidator.afterCheck(vo);

            return insert;
        });

        return result == null ? 0 : result;
    }

    /**
     * 更新订单产量表
     */
    private int batchUpdateOrderProductList(ReportBO bo) {
        if (bo == null || bo.getReportId() == null || CollectionUtils.isEmptyElement(bo.getProductList())) {
            return 0;
        }
        // 设置总产量ID
        for (ReportProdBO prod : bo.getProductList()) {
            if (prod.getUserProdList() == null) {
                continue;
            }
            for (ReportOrderProdBO userProd : prod.getOrderProdList()) {
                userProd.setReportProdId(prod.getId());
            }
        }

        // 扁平化，获取列表
        List<ReportOrderProdBO> newList = bo.getProductList().stream()
                .map(ReportProdBO::getOrderProdList)
                .flatMap(List::stream)
                .filter(Objects::nonNull)
                .collect(Collectors.toList());

        // 查询旧列表
        List<ReportOrderProdVO> oldList = reportOrderProdService.selectListByReportId(bo.getReportId());

        // 获取差异
        DiffListVO<ReportOrderProdBO, ReportOrderProdVO> diff = CollectionUtils.convertToDiffList(newList, oldList, ReportOrderProd::getId);

        int result = 0;
        if (diff.getAddCount() > 0) {
            int insert = reportOrderProdService.batchInsert(diff.getAddList());
            ServiceUtil.assertion(insert != diff.getAddCount(), "新增订单产量明细失败");
            result += insert;
        }
        if (diff.getUpdateCount() > 0) {
            int update = reportOrderProdService.batchUpdate(diff.getUpdateList());
            ServiceUtil.assertion(update != diff.getUpdateCount(), "修改订单产量明细失败");
            result += update;
        }
        if (diff.getDelCount() > 0) {
            int del = reportOrderProdService.batchLogicDel(diff.getDelList());
            ServiceUtil.assertion(del != diff.getDelCount(), "删除订单产量明细失败");
            result += del;
        }

        return result;
    }

    /**
     * 更新员工产量表
     * @param bo
     * @return
     */
    private int batchUpdateUserProductList(ReportBO bo) {
        if (bo == null || bo.getReportId() == null || CollectionUtils.isEmptyElement(bo.getProductList())) {
            return 0;
        }
        // 设置总产量ID
        for (ReportProdBO prod : bo.getProductList()) {
            if (prod.getUserProdList() == null) {
                continue;
            }
            for (ReportUserProdBO userProd : prod.getUserProdList()) {
                userProd.setProdId(prod.getId());
            }
        }

        // 扁平化，获取列表
        List<ReportUserProdBO> newList = bo.getProductList().stream()
                .map(ReportProdBO::getUserProdList)
                .flatMap(List::stream)
                .filter(Objects::nonNull)
                .collect(Collectors.toList());

        // 查询旧列表
        List<ReportUserProdVO> oldList = reportUserProdService.selectListByReportId(bo.getReportId());

        // 获取差异
        DiffListVO<ReportUserProdBO, ReportUserProdVO> diff = CollectionUtils.convertToDiffList(newList, oldList, ReportUserProd::getId);

        int result = 0;
        if (diff.getAddCount() > 0) {
            int insert = reportUserProdService.batchInsert(diff.getAddList());
            ServiceUtil.assertion(insert != diff.getAddCount(), "新增用户产量明细失败");
            result += insert;
        }
        if (diff.getUpdateCount() > 0) {
            int update = reportUserProdService.batchUpdate(diff.getUpdateList());
            ServiceUtil.assertion(update != diff.getUpdateCount(), "修改用户产量明细失败");
            result += update;
        }
        if (diff.getDelCount() > 0) {
            int del = reportUserProdService.batchLogicDel(diff.getDelList());
            ServiceUtil.assertion(del != diff.getDelCount(), "删除用户产量明细失败");
            result += del;
        }

        return result;
    }

    @Override
    public int editReport(ReportBO bo) {
        // 校验
        reportValidator.checkPreEdit(bo);
        ReportVO old = bo.getOld();

        Integer result = transactionTemplate.execute(status -> {
            // 若原来是审核未通过，则重置审核信息
            if (ReportStatus.REJECT.getStatus().equals(old.getStatus())) {
                int reset = this.resetVerifyInfo(bo.getReportId());
                ServiceUtil.assertion(reset != 1, "重置审核信息失败");
            }

            // 修改主表
            int update = this.updateReport(bo);
            ServiceUtil.assertion(update != 1, "修改报表失败");

            // 修改产量表
            this.batchUpdateProductList(bo);

            // 修改员工产量表
            this.batchUpdateUserProductList(bo);

            // 修改订单产量表
            this.batchUpdateOrderProductList(bo);

            // 后校验
            ReportVO vo = this.selectReportByReportId(bo.getReportId());
            reportAssembler.assembleProductList(Collections.singletonList(vo), true, true);
            reportValidator.afterCheck(vo);

            return update;
        });

        return result == null ? 0 : result;
    }

    private int resetVerifyInfo(Long reportId) {
        if (reportId == null) {
            return 0;
        }
        ReportQuery query = new ReportQuery();
        query.setReportId(reportId);
        query.setStatusList(ReportStatus.canResetVerify());
        return reportMapper.resetVerifyInfoByQuery(query);
    }

    @Override
    public int verify(ReportVerifyDTO dto, Long verifyId) {
        if (dto == null || dto.getReportId() == null) {
            return 0;
        }

        ReportVO report = selectReportByReportId(dto.getReportId());
        ServiceUtil.assertion(report == null, "待审核的报表不存在");
        ServiceUtil.assertion(!ReportStatus.canVerify().contains(report.getStatus()), "当前报表状态不允许审核，请刷新后重试");
        // 拼接报表订单数据
        reportAssembler.assembleProductList(Collections.singletonList(report), false, true);

        // 组装修改数据
        Report data = new Report();
        SysUser verifyUser = userService.selectUserById(verifyId);  // 审核人
        boolean pass = dto.getPass() != null && dto.getPass();
        data.setStatus(pass ? ReportStatus.PASS.getStatus() : ReportStatus.REJECT.getStatus());
        data.setVerifyRemark(dto.getVerifyRemark());
        data.setVerifyTime(LocalDateTime.now());
        if (verifyUser != null)  {
            data.setVerifyBy(verifyUser.getNickName());
            data.setVerifyId(verifyUser.getUserId());
        }
        // 组装查询条件
        ReportQuery query = new ReportQuery();
        query.setReportId(report.getReportId());
        query.setStatusList(ReportStatus.canVerify());

        // 获取订单数据
        List<ReportOrderProdVO> orderProdList = report.getProductList().stream()
                .map(ReportProdVO::getOrderProdList).flatMap(List::stream).collect(Collectors.toList());

        // 修改数据库
        Integer result = transactionTemplate.execute(status -> {
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "审核失败，当前报表状态已发生变化，请刷新后重试");

            // 关联的订单，全部增加已审核的数量
            for (ReportOrderProdVO orderProd : orderProdList) {
                if (orderProd == null) {
                    continue;
                }
                ServiceUtil.assertion(orderProd.getOrderId() == null
                        || orderProd.getNum() == null
                        || orderProd.getPriceQuantityNumerator() == null
                        || orderProd.getPriceQuantityDenominator() == null,
                        "修改订单审核数量出错：参数错误"
                );
                BigDecimal baseNum = ReportUtils.calcOrderProdBaseNum(orderProd);
                int add = prodOrderService.addVerifiedBaseNum(orderProd.getOrderId(), baseNum);
                ServiceUtil.assertion(add != 1, "更新关联订单失败");
            }

            return update;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int cancel(Long reportId) {
        ReportVO old = this.selectReportByReportId(reportId);
        ServiceUtil.assertion(old == null || old.getReportId() == null, "待取消的报表不存在");
        ServiceUtil.assertion(!ReportStatus.canCancel().contains(old.getStatus()), "报表当前状态不允许取消，请刷新后重试");

        Report data = new Report();
        data.setStatus(ReportStatus.WAIT_SUBMIT.getStatus());
        ReportQuery query = new ReportQuery();
        query.setReportId(reportId);
        query.setStatusList(ReportStatus.canCancel());
        Integer result = transactionTemplate.execute(status -> {
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "取消失败，当前报表状态已发生变化，请刷新后重试");
            return update;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int submit(Long reportId) {
        ReportVO old = this.selectReportByReportId(reportId);
        ServiceUtil.assertion(old == null || old.getReportId() == null, "待提交的报表不存在");
        ServiceUtil.assertion(!ReportStatus.canSubmit().contains(old.getStatus()), "报表当前状态不允许提交，请刷新后重试");

        Report data = new Report();
        data.setStatus(ReportStatus.WAIT_VERIFY.getStatus());
        ReportQuery query = new ReportQuery();
        query.setReportId(reportId);
        query.setStatusList(ReportStatus.canSubmit());
        Integer result = transactionTemplate.execute(status -> {
            int update = this.updateByQuery(data, query);
            ServiceUtil.assertion(update != 1, "提交失败，当前报表状态已发生变化，请刷新后重试");
            return update;
        });

        return result == null ? 0 : result;
    }

    @Override
    public int logicDel(List<Long> reportIds) {
        if (CollectionUtils.isEmptyElement(reportIds)) {
            return 0;
        }
        List<ReportVO> reportList = this.selectByReportIds(reportIds);

        int result = 0;
        for (Long reportId : reportIds) {
            try {
                ReportVO report = reportList.stream()
                        .filter(item -> Objects.equals(reportId, item.getReportId()))
                        .findFirst().orElse(null);
                int del = this.logicDel(report);
                ServiceUtil.assertion(del != 1, "删除报表失败");
                result += del;
            } catch (Exception e) {
                ServiceUtil.assertion(true, "删除ID为%s的报表失败，请重试：", reportId, e.getMessage());
            }
        }
        return result;
    }

    @Override
    public List<ReportVO> selectByReportIds(List<Long> reportIds) {
        if (CollectionUtils.isEmptyElement(reportIds)) {
            return Collections.emptyList();
        }

        ReportQuery query = new ReportQuery();
        query.setReportIds(reportIds);
        return this.selectReportList(query);
    }

    private int logicDel(ReportVO report) {
        ServiceUtil.assertion(report == null || report.getReportId() == null, "报表不存在");
        ServiceUtil.assertion(!ReportStatus.canDel().contains(report.getStatus()), "报表当前状态不允许删除");

        Integer result = transactionTemplate.execute(status -> {
            ReportQuery query = new ReportQuery();
            query.setReportId(report.getReportId());
            query.setStatusList(ReportStatus.canDel());
            int del = reportMapper.logicDelByQuery(query);

            if (del == 1) {
                // 删除产量报表
                reportProdService.logicDelByReportId(report.getReportId());

                // 删除订单报表
                reportOrderProdService.logicDelByReportId(report.getReportId());

                // 删除员工报表
                reportUserProdService.logicDelByReportId(report.getReportId());
            }

            return del;
        });

        return result == null ? 0 : result;
    }

    private int updateByQuery(Report data, ReportQuery query) {
        return reportMapper.updateByQuery(data, query);
    }

    /**
     * 更新产量表
     */
    private int batchUpdateProductList(ReportBO bo) {
        if (bo == null || CollectionUtils.isEmptyElement(bo.getProductList())) {
            return 0;
        }
        // 修改报表ID
        for (ReportProd prod : bo.getProductList()) {
            prod.setReportId(bo.getReportId());
        }

        List<ReportProdVO> oldList = reportProdService.selectByReportId(bo.getReportId());

        // 分离出新增、修改、删除的列表
        DiffListVO<ReportProdBO, ReportProdVO> diff = CollectionUtils.convertToDiffList(bo.getProductList(), oldList, ReportProd::getId);

        int result = 0;
        if (diff.getAddCount() > 0) {
            int batchInsert = reportProdService.batchInsert(diff.getAddList());
            ServiceUtil.assertion(batchInsert != diff.getAddCount(), "新增报表产量明细失败");
            result += batchInsert;
        }
        if (diff.getUpdateCount() > 0) {
            int update = reportProdService.batchUpdate(diff.getUpdateList());
            ServiceUtil.assertion(update != diff.getUpdateCount(), "修改报表产量明细失败");
            if (update > 0) {
                // 重置priceId 为 null 的数据
                List<Long> prodIds = diff.getUpdateList().stream().filter(item -> item.getPriceId() == null).map(ReportProd::getId).collect(Collectors.toList());
                reportProdService.resetPriceId(prodIds);
            }
            result += update;
        }
        if (diff.getDelCount() > 0) {
            int del = reportProdService.batchLogicDel(diff.getDelList());
            ServiceUtil.assertion(del != diff.getDelCount(), "删除报表产量明细失败");
            result += del;
        }

        return result;
    }

    @Override
    public int recalculateReportTotalAmountByPriceId(List<Long> priceIds) {
        if (CollectionUtils.isEmptyElement(priceIds)) {
            return 0;
        }
        ReportQuery query = new ReportQuery();
        query.setPriceIds(priceIds);
        return reportMapper.calcTotalAmount(query);
    }
}
