package com.ruoyi.yh.unit.service.impl;

import com.alibaba.fastjson2.JSONArray;
import com.github.pagehelper.PageHelper;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.core.redis.enums.RedisLockKey;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.k3cloud.constants.K3FormIds;
import com.ruoyi.common.k3cloud.constants.fileds.K3UnitField;
import com.ruoyi.common.k3cloud.service.K3Service;
import com.ruoyi.common.utils.ServiceUtil;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.yh.logImport.domain.LogImportVO;
import com.ruoyi.yh.logImport.domain.enums.LogImportBizType;
import com.ruoyi.yh.logImport.domain.enums.LogImportType;
import com.ruoyi.yh.logImport.service.LogImportService;
import com.ruoyi.yh.logImportDetail.domain.enums.LogImportDetailType;
import com.ruoyi.yh.logImportDetail.service.ILogImportDetailService;
import com.ruoyi.yh.unit.domain.Unit;
import com.ruoyi.yh.unit.domain.UnitQuery;
import com.ruoyi.yh.unit.domain.UnitVO;
import com.ruoyi.yh.unit.mapper.UnitMapper;
import com.ruoyi.yh.unit.service.UnitConverter;
import com.ruoyi.yh.unit.service.UnitService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 单位Service业务层处理
 *
 * @author ruoyi
 * @date 2024-11-18
 */
@Service
@Slf4j
public class UnitServiceImpl implements UnitService
{
    @Autowired
    private UnitMapper unitMapper;

    @Autowired
    private K3Service k3Service;

    @Autowired
    private UnitConverter unitConverter;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private RedisLock redisLock;

    @Autowired
    private LogImportService logImportService;

    @Autowired
    private ILogImportDetailService logImportDetailService;

    @Autowired
    private TransactionTemplate transactionTemplate;

    /**
     * 查询单位
     *
     * @param unitId 单位主键
     * @return 单位
     */
    @Override
    public UnitVO selectUnitByUnitId(Long unitId)
    {
        return unitMapper.selectUnitByUnitId(unitId);
    }

    /**
     * 查询单位列表
     *
     * @param unit 单位
     * @return 单位
     */
    @Override
    public List<UnitVO> selectUnitList(UnitQuery unit)
    {
        return unitMapper.selectUnitList(unit);
    }

    /**
     * 新增单位
     *
     * @param unit 单位
     * @return 结果
     */
    @Override
    public int insertUnit(Unit unit)
    {
        return unitMapper.insertUnit(unit);
    }

    /**
     * 修改单位
     *
     * @param unit 单位
     * @return 结果
     */
    @Override
    public int updateUnit(Unit unit)
    {
        return unitMapper.updateUnit(unit);
    }

    /**
     * 批量删除单位
     *
     * @param unitIds 需要删除的单位主键
     * @return 结果
     */
    @Override
    public int deleteUnitByUnitIds(Long[] unitIds)
    {
        return unitMapper.deleteUnitByUnitIds(unitIds);
    }

    /**
     * 删除单位信息
     *
     * @param unitId 单位主键
     * @return 结果
     */
    @Override
    public int deleteUnitByUnitId(Long unitId)
    {
        return unitMapper.deleteUnitByUnitId(unitId);
    }

    @Override
    public void sync(Long operaId) {
        // 新增一个日志
        LogImportVO importLog = logImportService.addDefaultLog(LogImportType.SYNC, LogImportBizType.UNIT, operaId);
        ServiceUtil.assertion(importLog == null, "创建同步日志失败");
        Long logId = importLog.getLogId();

        // 查询字段
        scheduledExecutorService.schedule(() -> {
            List<String> fieldKeys = Arrays.asList(
                K3UnitField.F_UNIT_ID,
                K3UnitField.F_NUMBER,
                K3UnitField.F_NAME,
                K3UnitField.F_FORBID_STATUS
            );
            int startRow = 0;
            int limit = 10000;
            int size = 0;

            // 一直查询，直到查到的数据量比limit小，说明查完了
            while(true) {
                // 查询ERP数据
                List<JSONArray> erpList = k3Service.selectList(K3FormIds.UNIT, fieldKeys, startRow, limit);

                // 转为PO
                List<Unit> unitList = unitConverter.toPoByErpList(fieldKeys, erpList);

                // 保存
                for (Unit unit : unitList) {
                    scheduledExecutorService.schedule(() -> {
                        this.saveByErpId(unit, logId);
                    }, 0, TimeUnit.SECONDS);
                }

                size += unitList.size();

                if (unitList.size() < limit) {
                    logImportService.updateLogToFinished(logId);
                    break;
                } else {
                    startRow += limit;
                }
            }
        }, 0, TimeUnit.SECONDS);
    }

    @Override
    public List<UnitVO> selectByErpIds(Collection<String> erpIds) {
        if (CollectionUtils.isEmptyElement(erpIds)) {
            return Collections.emptyList();
        }

        UnitQuery query = new UnitQuery();
        query.setErpIds(erpIds);
        return this.selectUnitList(query);
    }

    private int saveByErpId(Unit unit, Long logId) {
        if (unit == null || StringUtils.isBlank(unit.getErpId())) {
            throw new ServiceException("参数错误");
        }
        String lockKey = unit.getErpId();
        ServiceUtil.assertion(!redisLock.lock(RedisLockKey.SAVE_UNIT, lockKey), "当前单位正在处理中，请稍后重试");
        try {
            // 若数据库中没有,则新增,否则修改
            UnitVO old = this.selectUnitByErpId(unit.getErpId());

            Integer result = transactionTemplate.execute(status -> {
                if (old == null) {
                    int insert = this.insertUnit(unit);
                    ServiceUtil.assertion(insert != 1, "新增物料失败");
                    int log = logImportDetailService.addResult(logId, LogImportDetailType.ADD, unit, null, unit.getUnitId());
                    ServiceUtil.assertion(log != 1, "新增日志失败");
                    return insert;
                } else {
                    unit.setUnitId(old.getUnitId());
                    int update = this.updateUnit(unit);
                    ServiceUtil.assertion(update != 1, "修改物料失败");
                    int log = logImportDetailService.addResult(logId, LogImportDetailType.UPDATE, unit, null, unit.getUnitId());
                    ServiceUtil.assertion(log != 1, "新增日志失败");
                    return update;
                }
            });

            return result == null ? 0 : result;
        } catch (Exception e) {
            logImportDetailService.addResult(logId, LogImportDetailType.FAIL, unit, e.getMessage(), unit.getUnitId());
            return 0;
        } finally {
            redisLock.unlock(RedisLockKey.SAVE_UNIT, lockKey);
        }
    }

    private UnitVO selectUnitByErpId(String erpId) {
        if (StringUtils.isBlank(erpId)) {
            return null;
        }
        UnitQuery query = new UnitQuery();
        query.setErpId(erpId);
        return this.selectOne(query);
    }

    private UnitVO selectOne(UnitQuery query) {
        PageHelper.startPage(1, 1);
        List<UnitVO> list = this.selectUnitList(query);
        if (CollectionUtils.isEmpty(list)) {
            return null;
        }
        return list.get(0);
    }
}
